#!/bin/bash

# SPDX-FileCopyrightText: 2009 Fermi Research Alliance, LLC
# SPDX-License-Identifier: Apache-2.0

# All possible hooks:
# HOOK_NAMES="applypatch-msg pre-applypatch post-applypatch pre-commit prepare-commit-msg commit-msg post-commit pre-rebase post-checkout post-merge pre-receive update post-receive post-update pre-auto-gc"
# Link only the hooks used
HOOK_NAMES="pre-commit post-checkout"

ROOT_DIR=$(git rev-parse --show-toplevel)
HOOK_DIR="$ROOT_DIR"/.git/hooks
# Relative path, assuming the script is in build/bigfiles directory
MY_DIR=build/bigfiles
WRAPPER_PATH="$MY_DIR"/hooks-wrapper

# hooks-wrapper is counting on a tracked_hooks subdirectory
ln -s -f ../../"$MY_DIR"/hooks "$HOOK_DIR"/tracked_hooks

for hook in $HOOK_NAMES; do
    # If the hook already exists, is executable, and is not a symlink
    if [ ! -h "$HOOK_DIR/$hook" -a -x "$HOOK_DIR/$hook" ]; then
        mv "$HOOK_DIR/$hook" "$HOOK_DIR/$hook.local"
    fi
    # create the symlink, overwriting the file if it exists
    # probably the only way this would happen is if you're using an old version of git
    # -- back when the sample hooks were not executable, instead of being named ____.sample
    # or if the script is re-executed
    ln -s -f ../../"$WRAPPER_PATH" "$HOOK_DIR/$hook"
done
